@@ -3,6 +3,7 @@ from __future__ import unicode_literals
 from __future__ import print_function
 
 import os
+import os.path
 import sys
 import traceback
 import logging
@@ -15,8 +16,9 @@ from io import open
 import click
 import sqlparse
 from prompt_toolkit import CommandLineInterface, Application, AbortAction
+from prompt_toolkit.interface import AcceptAction
 from prompt_toolkit.enums import DEFAULT_BUFFER
-from prompt_toolkit.shortcuts import create_default_layout, create_eventloop
+from prompt_toolkit.shortcuts import create_prompt_layout, create_eventloop
 from prompt_toolkit.document import Document
 from prompt_toolkit.filters import Always, HasFocus, IsDone
 from prompt_toolkit.layout.processors import (HighlightMatchingBracketProcessor,
@@ -35,8 +37,9 @@ from .clistyle import style_factory
 from .sqlexecute import SQLExecute
 from .clibuffer import CLIBuffer
 from .completion_refresher import CompletionRefresher
-from .config import (write_default_config, load_config, get_mylogin_cnf_path,
-                     open_mylogin_cnf, CryptoError)
+from .config import (write_default_config, get_mylogin_cnf_path,
+                     open_mylogin_cnf, CryptoError, read_config_file,
+                     read_config_files, str_to_bool)
 from .key_bindings import mycli_bindings
 from .encodingutils import utf8tounicode
 from .lexer import MyCliLexer
@@ -67,16 +70,25 @@ class MyCli(object):
         '/etc/my.cnf',
         '/etc/mysql/my.cnf',
         '/usr/local/etc/my.cnf',
-        os.path.expanduser('~/.my.cnf')
+        '~/.my.cnf'
     ]
 
+    system_config_files = [
+		'/etc/myclirc',
+    ]
+
+    default_config_file = os.path.join(PACKAGE_ROOT, 'myclirc')
+    user_config_file = '~/.myclirc'
+
+
     def __init__(self, sqlexecute=None, prompt=None,
             logfile=None, defaults_suffix=None, defaults_file=None,
-            login_path=None):
+            login_path=None, auto_vertical_output=False):
         self.sqlexecute = sqlexecute
         self.logfile = logfile
         self.defaults_suffix = defaults_suffix
         self.login_path = login_path
+        self.auto_vertical_output = auto_vertical_output
 
         # self.cnf_files is a class variable that stores the list of mysql
         # config files to read in at launch.
@@ -85,12 +97,10 @@ class MyCli(object):
         if defaults_file:
             self.cnf_files = [defaults_file]
 
-        default_config = os.path.join(PACKAGE_ROOT, 'myclirc')
-        write_default_config(default_config, '~/.myclirc')
-
-
         # Load config.
-        c = self.config = load_config('~/.myclirc', default_config)
+        config_files = ([self.default_config_file] + self.system_config_files +
+                        [self.user_config_file])
+        c = self.config = read_config_files(config_files)
         self.multi_line = c['main'].as_bool('multi_line')
         self.destructive_warning = c['main'].as_bool('destructive_warning')
         self.key_bindings = c['main']['key_bindings']
@@ -100,6 +110,10 @@ class MyCli(object):
         self.cli_style = c['colors']
         self.wider_completion_menu = c['main'].as_bool('wider_completion_menu')
 
+        # Write user config if system config wasn't the last config loaded.
+        if c.filename not in self.system_config_files:
+            write_default_config(self.default_config_file, self.user_config_file)
+
         # audit log
         if self.logfile is None and 'audit_log' in c['main']:
             try:
@@ -225,14 +239,16 @@ class MyCli(object):
         root_logger.addHandler(handler)
         root_logger.setLevel(level_map[log_level.upper()])
 
+        logging.captureWarnings(True)
+
         root_logger.debug('Initializing mycli logging.')
         root_logger.debug('Log file %r.', log_file)
 
-    def connect_uri(self, uri):
+    def connect_uri(self, uri, local_infile=None, ssl=None):
         uri = urlparse(uri)
         database = uri.path[1:]  # ignore the leading fwd slash
         self.connect(database, uri.username, uri.password, uri.hostname,
-                uri.port)
+                uri.port, local_infile=local_infile, ssl=ssl)
 
     def read_my_cnf_files(self, files, keys):
         """
@@ -241,15 +257,7 @@ class MyCli(object):
         :param keys: list of keys to retrieve
         :returns: tuple, with None for missing keys.
         """
-        cnf = ConfigObj()
-        for _file in files:
-            try:
-                cnf.merge(ConfigObj(_file, interpolation=False))
-            except ConfigObjError as e:
-                self.logger.error('Error parsing %r.', _file)
-                self.logger.error('Recovering partially parsed config values.')
-                cnf.merge(e.config)
-                pass
+        cnf = read_config_files(files)
 
         sections = ['client']
         if self.login_path and self.login_path != 'client':
@@ -267,8 +275,31 @@ class MyCli(object):
 
         return dict([(x, get(x)) for x in keys])
 
+    def merge_ssl_with_cnf(self, ssl, cnf):
+        """Merge SSL configuration dict with cnf dict"""
+
+        merged = {}
+        merged.update(ssl)
+        prefix = 'ssl-'
+        for k, v in cnf.items():
+            # skip unrelated options
+            if not k.startswith(prefix):
+                continue
+            if v is None:
+                continue
+            # special case because PyMySQL argument is significantly different
+            # from commandline
+            if k == 'ssl-verify-server-cert':
+                merged['check_hostname'] = v
+            else:
+                # use argument name just strip "ssl-" prefix
+                arg = k[len(prefix):]
+                merged[arg] = v
+
+        return merged
+
     def connect(self, database='', user='', passwd='', host='', port='',
-            socket='', charset=''):
+            socket='', charset='', local_infile='', ssl=''):
 
         cnf = {'database': None,
                'user': None,
@@ -276,7 +307,15 @@ class MyCli(object):
                'host': None,
                'port': None,
                'socket': None,
-               'default-character-set': None}
+               'default-character-set': None,
+               'local-infile': None,
+               'loose-local-infile': None,
+               'ssl-ca': None,
+               'ssl-cert': None,
+               'ssl-key': None,
+               'ssl-cipher': None,
+               'ssl-verify-serer-cert': None,
+        }
 
         cnf = self.read_my_cnf_files(self.cnf_files, cnf.keys())
 
@@ -289,22 +328,45 @@ class MyCli(object):
             socket = socket or cnf['socket']
         user = user or cnf['user'] or os.getenv('USER')
         host = host or cnf['host'] or 'localhost'
-        port = int(port or cnf['port'] or 3306)
+        port = port or cnf['port'] or 3306
+        ssl = ssl or {}
+
+        try:
+            port = int(port)
+        except ValueError as e:
+            self.output("Error: Invalid port number: '{0}'.".format(port),
+                        err=True, fg='red')
+            exit(1)
+
         passwd = passwd or cnf['password']
         charset = charset or cnf['default-character-set'] or 'utf8'
 
+        # Favor whichever local_infile option is set.
+        for local_infile_option in (local_infile, cnf['local-infile'],
+                                    cnf['loose-local-infile'], False):
+            try:
+                local_infile = str_to_bool(local_infile_option)
+                break
+            except (TypeError, ValueError):
+                pass
+
+        ssl = self.merge_ssl_with_cnf(ssl, cnf)
+        # prune lone check_hostname=False
+        if not any(v for v in ssl.values()):
+            ssl = None
+
         # Connect to the database.
 
         try:
             try:
                 sqlexecute = SQLExecute(database, user, passwd, host, port,
-                        socket, charset)
+                        socket, charset, local_infile, ssl)
             except OperationalError as e:
                 if ('Access denied for user' in e.args[1]):
                     passwd = click.prompt('Password', hide_input=True,
                                           show_default=False, type=str)
                     sqlexecute = SQLExecute(database, user, passwd, host, port,
-                            socket, charset)
+                            socket, charset, local_infile, ssl)
                 else:
                     raise e
         except Exception as e:  # Connecting to a database could fail.
@@ -343,8 +405,7 @@ class MyCli(object):
     def run_cli(self):
         sqlexecute = self.sqlexecute
         logger = self.logger
-        original_less_opts = self.adjust_less_opts()
-        self.set_pager_from_config()
+        self.configure_pager()
 
         self.refresh_completions()
 
@@ -368,29 +429,33 @@ class MyCli(object):
         def prompt_tokens(cli):
             return [(Token.Prompt, self.get_prompt(self.prompt_format))]
 
+        def get_continuation_tokens(cli, width):
+            return [(Token.Continuation, ' ' * (width - 3) + '-> ')]
+
         get_toolbar_tokens = create_toolbar_tokens_func(lambda: self.key_bindings,
                                                         self.completion_refresher.is_refreshing)
 
-        layout = create_default_layout(lexer=MyCliLexer,
-                                       reserve_space_for_menu=True,
-                                       multiline=True,
-                                       get_prompt_tokens=prompt_tokens,
-                                       get_bottom_toolbar_tokens=get_toolbar_tokens,
-                                       display_completions_in_columns=self.wider_completion_menu,
-                                       extra_input_processors=[
-                                           ConditionalProcessor(
-                                               processor=HighlightMatchingBracketProcessor(chars='[](){}'),
-                                               filter=HasFocus(DEFAULT_BUFFER) & ~IsDone()),
-                                       ])
+        layout = create_prompt_layout(lexer=MyCliLexer,
+                                      multiline=True,
+                                      get_prompt_tokens=prompt_tokens,
+                                      get_continuation_tokens=get_continuation_tokens,
+                                      get_bottom_toolbar_tokens=get_toolbar_tokens,
+                                      display_completions_in_columns=self.wider_completion_menu,
+                                      extra_input_processors=[
+                                          ConditionalProcessor(
+                                              processor=HighlightMatchingBracketProcessor(chars='[](){}'),
+                                              filter=HasFocus(DEFAULT_BUFFER) & ~IsDone()),
+                                      ])
         with self._completer_lock:
             buf = CLIBuffer(always_multiline=self.multi_line, completer=self.completer,
                     history=FileHistory(os.path.expanduser('~/.mycli-history')),
-                    complete_while_typing=Always())
+                    complete_while_typing=Always(), accept_action=AcceptAction.RETURN_DOCUMENT)
 
             application = Application(style=style_factory(self.syntax_style, self.cli_style),
                                       layout=layout, buffer=buf,
                                       key_bindings_registry=key_binding_manager.registry,
                                       on_exit=AbortAction.RAISE_EXCEPTION,
+                                      on_abort=AbortAction.RETRY,
                                       ignore_case=True)
             self.cli = CommandLineInterface(application=application,
                                        eventloop=create_eventloop())
@@ -456,14 +521,23 @@ class MyCli(object):
                             if not click.confirm('Do you want to continue?'):
                                 self.output("Aborted!", err=True, fg='red')
                                 break
-                        output.extend(format_output(title, cur, headers,
-                            status, self.table_format))
+
+                        if self.auto_vertical_output:
+                            max_width = self.cli.output.get_size().columns
+                        else:
+                            max_width = None
+
+                        formatted = format_output(title, cur, headers,
+                            status, self.table_format,
+                            special.is_expanded_output(), max_width)
+
+                        output.extend(formatted)
                         end = time()
                         total += end - start
                         mutating = mutating or is_mutating(status)
                 except UnicodeDecodeError as e:
                     import pymysql
-                    if pymysql.VERSION < ('0', '6', '7'):
+                    if pymysql.VERSION < (0, 6, 7):
                         message = ('You are running an older version of pymysql.\n'
                                 'Please upgrade to 0.6.7 or above to view binary data.\n'
                                 'Try \'pip install -U pymysql\'.')
@@ -505,7 +579,10 @@ class MyCli(object):
                     self.output(str(e), err=True, fg='red')
                 else:
                     try:
-                        self.output_via_pager('\n'.join(output))
+                        if special.is_pager_enabled():
+                            self.output_via_pager('\n'.join(output))
+                        else:
+                            self.output('\n'.join(output))
                     except KeyboardInterrupt:
                         pass
                     if special.is_timing_enabled():
@@ -523,10 +600,6 @@ class MyCli(object):
 
         except EOFError:
             self.output('Goodbye!')
-        finally:  # Reset the less opts back to original.
-            logger.debug('Restoring env var LESS to %r.', original_less_opts)
-            os.environ['LESS'] = original_less_opts
-            os.environ['PAGER'] = special.get_original_pager()
 
     def output(self, text, **kwargs):
         if self.logfile:
@@ -540,17 +613,15 @@ class MyCli(object):
             self.logfile.write('\n')
         click.echo_via_pager(text)
 
-    def adjust_less_opts(self):
-        less_opts = os.environ.get('LESS', '')
-        self.logger.debug('Original value for LESS env var: %r', less_opts)
-        os.environ['LESS'] = '-SRXF'
+    def configure_pager(self):
+        # Provide sane defaults for less.
+        os.environ['LESS'] = '-RXF'
 
-        return less_opts
-
-    def set_pager_from_config(self):
-        cnf = self.read_my_cnf_files(self.cnf_files, ['pager'])
+        cnf = self.read_my_cnf_files(self.cnf_files, ['pager', 'skip-pager'])
         if cnf['pager']:
             special.set_pager(cnf['pager'])
+        if cnf['skip-pager']:
+            special.disable_pager()
 
     def refresh_completions(self, reset=False):
         if reset:
@@ -605,6 +676,20 @@ class MyCli(object):
               help='Password to connect to the database')
 @click.option('--pass', 'password', envvar='MYSQL_PWD', type=str,
               help='Password to connect to the database')
+@click.option('--ssl-ca', help='CA file in PEM format',
+              type=click.Path(exists=True))
+@click.option('--ssl-capath', help='CA directory')
+@click.option('--ssl-cert', help='X509 cert in PEM format',
+              type=click.Path(exists=True))
+@click.option('--ssl-key', help='X509 key in PEM format',
+              type=click.Path(exists=True))
+@click.option('--ssl-cipher', help='SSL cipher to use')
+@click.option('--ssl-verify-server-cert', is_flag=True,
+              help=('Verify server\'s "Common Name" in its cert against '
+                    'hostname used when connecting. This option is disabled '
+                    'by default'))
+# as of 2016-02-15 revocation list is not supported by underling PyMySQL
+# library (--ssl-crl and --ssl-crlpath options in vanilla mysql client)
 @click.option('-v', '--version', is_flag=True, help='Version of mycli.')
 @click.option('-D', '--database', 'dbname', help='Database to use.')
 @click.option('-R', '--prompt', 'prompt',
@@ -616,27 +701,45 @@ class MyCli(object):
               help='Read config group with the specified suffix.')
 @click.option('--defaults-file', type=click.Path(),
               help='Only read default options from the given file')
+@click.option('--auto-vertical-output', is_flag=True,
+              help='Automatically switch to vertical output mode if the result is wider than the terminal width.')
+@click.option('--local-infile', type=bool,
+              help='Enable/disable LOAD DATA LOCAL INFILE.')
 @click.option('--login-path', type=str,
               help='Read this path from the login file.')
 @click.argument('database', default='', nargs=1)
 def cli(database, user, host, port, socket, password, dbname,
         version, prompt, logfile, defaults_group_suffix, defaults_file,
-        login_path):
+        login_path, auto_vertical_output, local_infile, ssl_ca, ssl_capath,
+        ssl_cert, ssl_key, ssl_cipher, ssl_verify_server_cert):
     if version:
         print('Version:', __version__)
         sys.exit(0)
 
     mycli = MyCli(prompt=prompt, logfile=logfile,
                   defaults_suffix=defaults_group_suffix,
-                  defaults_file=defaults_file, login_path=login_path)
+                  defaults_file=defaults_file, login_path=login_path,
+                  auto_vertical_output=auto_vertical_output)
 
     # Choose which ever one has a valid value.
     database = database or dbname
 
+    ssl = {
+            'ca': ssl_ca and os.path.expanduser(ssl_ca),
+            'cert': ssl_cert and os.path.expanduser(ssl_cert),
+            'key': ssl_key and os.path.expanduser(ssl_key),
+            'capath': ssl_capath,
+            'cipher': ssl_cipher,
+            'check_hostname': ssl_verify_server_cert,
+            }
+
+    # remove empty ssl options
+    ssl = dict((k, v) for (k, v) in ssl.items() if v is not None)
     if database and '://' in database:
-        mycli.connect_uri(database)
+        mycli.connect_uri(database, local_infile, ssl)
     else:
-        mycli.connect(database, user, password, host, port, socket)
+        mycli.connect(database, user, password, host, port, socket,
+                      local_infile=local_infile, ssl=ssl)
 
     mycli.logger.debug('Launch Params: \n'
             '\tdatabase: %r'
@@ -646,21 +749,35 @@ def cli(database, user, host, port, socket, password, dbname,
 
     mycli.run_cli()
 
-def format_output(title, cur, headers, status, table_format):
+def format_output(title, cur, headers, status, table_format, expanded=False, max_width=None):
     output = []
     if title:  # Only print the title if it's not None.
         output.append(title)
     if cur:
         headers = [utf8tounicode(x) for x in headers]
-        if special.is_expanded_output():
+        if expanded:
             output.append(expanded_table(cur, headers))
         else:
-            output.append(tabulate(cur, headers, tablefmt=table_format,
-                missingval='<null>'))
+            rows = list(cur)
+            tabulated, frows = tabulate(rows, headers, tablefmt=table_format,
+                missingval='<null>')
+            if (max_width and rows and
+                    content_exceeds_width(frows[0], max_width) and
+                    headers):
+                output.append(expanded_table(rows, headers))
+            else:
+                output.append(tabulated)
     if status:  # Only print the status if it's not None.
         output.append(status)
     return output
 
+def content_exceeds_width(row, width):
+    # Account for 3 characters between each column
+    separator_space = (len(row)*3)
+    # Add 2 columns for a bit of buffer
+    line_len = sum([len(str(x)) for x in row]) + separator_space + 2
+    return line_len > width
+
 def need_completion_refresh(queries):
     """Determines if the completion needs a refresh by checking if the sql
     statement is an alter, create, drop or change db."""
